/*
 * RELIC is an Efficient LIbrary for Cryptography
 * Copyright (C) 2007-2012 RELIC Authors
 *
 * This file is part of RELIC. RELIC is legal property of its developers,
 * whose names are not listed here. Please refer to the COPYRIGHT file
 * for contact information.
 *
 * RELIC is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * RELIC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file
 *
 * Interface of the low-level block cipher module.
 *
 * @version $Id$
 * @ingroup bc
 */

#ifndef RELIC_BC_LOW_H
#define RELIC_BC_LOW_H

/*
 * Provides useful constants for Assembly backends.
 */
#ifdef ASM

#include "relic_conf.h"

#else

/**
 * Computes the xor of two blocks.
 *
 * Each block must have BC_BLOCK_LEN bytes and be aligned.
 *
 * @param[out] c			- the result.
 * @param[in] a				- the first block to xor.
 * @param[in] b				- the second block to xor.
 */
void bc_xor_low(bc_t c, bc_t a, bc_t b);

/**
 * Computes the xor of two buffers.
 *
 * @param[out] c			- the result.
 * @param[in] a				- the first buffer to xor.
 * @param[in] b				- the second buffer to xor.
 * @param[in] len			- the length of the buffers.
 */
void bc_nxor_low(unsigned char *c, unsigned char *a, unsigned char *b, size_t len);

/**
 * Increments the last (most significant) 4 bytes of a block, viewed as a
 * big-endian integer.
 *
 * @param[in,out] block			- the block to increment.
 */
void bc_inc32_low(bc_t block);

#endif /* !ASM */

#endif /* !RELIC_BC_LOW_H */
