/*
 * RELIC is an Efficient LIbrary for Cryptography
 * Copyright (C) 2007-2012 RELIC Authors
 *
 * This file is part of RELIC. RELIC is legal property of its developers,
 * whose names are not listed here. Please refer to the COPYRIGHT file
 * for contact information.
 *
 * RELIC is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * RELIC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file
 *
 * Implementation of the low-level authenticated cipher CCFBH functions.
 *
 * @version $Id$
 * @ingroup ac
 */

#include <string.h>

#include "relic_ac.h"
#include "relic_ac_low.h"

/*============================================================================*/
/* Public definitions                                                         */
/*============================================================================*/

void ac_ccfbh_xor_low(unsigned char *c, unsigned char *a, unsigned char *b) {
	dig_t *pc = (dig_t *) c;
	dig_t *pa = (dig_t *) a;
	dig_t *pb = (dig_t *) b;
	int i;

	for (i = 0; i < AC_CCFBH_BLOCK_LEN / sizeof(dig_t); i++, pa++, pb++, pc++) {
		(*pc) = (*pa) ^ (*pb);
	}
}

void ac_ccfbh_xortag_low(unsigned char *c, unsigned char *a) {
	dig_t *pc = (dig_t *) c;
	dig_t *pa = (dig_t *) a;
	int i;

	for (i = 0; i < AC_CCFBH_TAG_LEN / sizeof(dig_t); i++, pa++, pc++) {
		(*pc) = (*pc) ^ (*pa);
	}
}

void ac_ccfbh_copy_low(unsigned char *c, unsigned char *a) {
	memcpy(c, a, AC_CCFBH_BLOCK_LEN);
}
