/*
 * RELIC is an Efficient LIbrary for Cryptography
 * Copyright (C) 2007-2012 RELIC Authors
 *
 * This file is part of RELIC. RELIC is legal property of its developers,
 * whose names are not listed here. Please refer to the COPYRIGHT file
 * for contact information.
 *
 * RELIC is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * RELIC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file
 *
 * Implementation of the low-level authenticated cipher OCB3 functions.
 *
 * @version $Id$
 * @ingroup ac
 */

#include <string.h>

#include "relic_ac.h"
#include "relic_ac_low.h"
#include "relic_types.h"
#include "relic_util.h"

/*============================================================================*/
/* Private definitions                                                        */
/*============================================================================*/

static int lsh(dig_t *c, dig_t *a, int size, int bits) {
	int i;
	dig_t r, carry, shift, mask;

	shift = DIGIT - bits;
	carry = 0;
	mask = MASK(bits);
	a += size - 1;
	c += size - 1;
	for (i = size - 1; i >= 0; i--, a--, c--) {
		/* Get the needed least significant bits. */
		r = ((*a) >> shift) & mask;
		/* Shift left the operand. */
		*c = ((*a) << bits) | carry;
		/* Update the carry. */
		carry = r;
	}
	return carry;
}

/*============================================================================*/
/* Public definitions                                                         */
/*============================================================================*/

void ac_ocb3_dbl_low(bc_t b) {
	int carry;
	carry = lsh((dig_t *) b, (dig_t *) b, BC_BLOCK_LEN / sizeof(dig_t), 1);
	if (carry) {
		((dig_t *) b)[BC_BLOCK_LEN / sizeof(dig_t) - 1] ^= 135u;
	}
}

unsigned int ac_ocb3_ntz_low(unsigned int v) {
	int c = 0;
	if (v) {
		v = (v ^ (v - 1)) >> 1; // Set v's trailing 0s to 1s and zero the rest
		for (c = 0; v; c++) {
			v >>= 1;
		}
	} else {
		c = 8 * sizeof(v);
	}
	return c;
}
