/*
 * RELIC is an Efficient LIbrary for Cryptography
 * Copyright (C) 2007-2012 RELIC Authors
 *
 * This file is part of RELIC. RELIC is legal property of its developers,
 * whose names are not listed here. Please refer to the COPYRIGHT file
 * for contact information.
 *
 * RELIC is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * RELIC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file
 *
 * Implementation of the low-level authenticated cipher GCM functions.
 *
 * @version $Id$
 * @ingroup ac
 */

#include <string.h>

#include "relic_ac.h"
#include "relic_ac_low.h"
#include "relic_types.h"
#include "relic_util.h"

#ifdef AC_GCM_SGCM

/*============================================================================*/
/* Private definitions                                                        */
/*============================================================================*/

#define GCM_DIGS (AC_GCM_BLOCK_LEN / sizeof(dig_t))

/**
 * Accumulates a double precision digit in a triple register variable.
 *
 * @param[in,out] R2		- most significant word of the triple register.
 * @param[in,out] R1		- middle word of the triple register.
 * @param[in,out] R0		- lowest significant word of the triple register.
 * @param[in] A				- the first digit to multiply.
 * @param[in] B				- the second digit to multiply.
 */
#define COMBA_STEP(R2, R1, R0, A, B)										\
	dbl_t r = (dbl_t)(A) * (dbl_t)(B);										\
	dig_t _r = (R1);														\
	(R0) += (dig_t)(r);														\
	(R1) += (R0) < (dig_t)(r);												\
	(R2) += (R1) < _r;														\
	(R1) += (dig_t)((r) >> (dbl_t)DIGIT);									\
	(R2) += (R1) < (dig_t)((r) >> (dbl_t)DIGIT);							\


void muln(dig_t *c, dig_t *a, dig_t *b) {
	int i, j;
	dig_t *tmpa, *tmpb, r0, r1, r2;

	r0 = r1 = r2 = 0;
	for (i = 0; i < GCM_DIGS; i++, c++) {
		tmpa = a;
		tmpb = b + i;
		for (j = 0; j <= i; j++, tmpa++, tmpb--) {
			COMBA_STEP(r2, r1, r0, *tmpa, *tmpb);
		}
		*c = r0;
		r0 = r1;
		r1 = r2;
		r2 = 0;
	}
	for (i = 0; i < GCM_DIGS; i++, c++) {
		tmpa = a + i + 1;
		tmpb = b + (GCM_DIGS - 1);
		for (j = 0; j < GCM_DIGS - (i + 1); j++, tmpa++, tmpb--) {
			COMBA_STEP(r2, r1, r0, *tmpa, *tmpb);
		}
		*c = r0;
		r0 = r1;
		r1 = r2;
		r2 = 0;
	}
}

void rdcn(dig_t *c, dig_t *a) {
	dig_t t[GCM_DIGS];
	dig_t d;

	//TODO: implement it :P
#error "SGCM is not implement yet in the easy backend"
}

/*============================================================================*/
/* Public definitions                                                         */
/*============================================================================*/

void ac_gcm_tab_low(dig_t *t, unsigned char *h) {
}

void ac_gcm_mul_low(dig_t *c, dig_t *a, dig_t *b) {
	dig_t t[GCM_DIGS*2];

	muln(t, a, b);
	rdcn(c, t);
}

#endif /* #ifndef AC_GCM_SGCM */
