/*
 * RELIC is an Efficient LIbrary for Cryptography
 * Copyright (C) 2007-2012 RELIC Authors
 *
 * This file is part of RELIC. RELIC is legal property of its developers,
 * whose names are not listed here. Please refer to the COPYRIGHT file
 * for contact information.
 *
 * RELIC is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * RELIC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file
 *
 * Implementation of the low-level authenticated cipher CCFBH functions.
 *
 * @version $Id$
 * @ingroup ac
 */

#include "relic_bc_conf.h"

#ifndef MSP430X
#define reta ret
#endif

NAME relic_bc_ccfbh_asm
RSEG CODE
EVEN

PUBLIC ccfbh_xor
PUBLIC ccfbh_copy
PUBLIC ccfbh_tag_xor

/*============================================================================*/
/* Public definitions                                                         */
/*============================================================================*/

RSEG CODE:CODE:REORDER:NOROOT(1)
ccfbh_xor:
    mov @r13+,r15
    xor @r14+,r15
    mov r15,2*0(r12)
    mov @r13+,r15
    xor @r14+,r15
    mov r15,2*1(r12)
    mov @r13+,r15
    xor @r14+,r15
    mov r15,2*2(r12)
    mov @r13+,r15
    xor @r14+,r15
    mov r15,2*3(r12)
#if (BC_CCFBH_TAG_LEN < 8)
    mov @r13+,r15
    xor @r14+,r15
    mov r15,2*4(r12)
#if (BC_CCFBH_TAG_LEN < 6)
    mov @r13+,r15
    xor @r14+,r15
    mov r15,2*5(r12)
#endif
#endif
    reta

RSEG CODE:CODE:REORDER:NOROOT(1)
ccfbh_copy:
    mov @r13+,2*0(r12)
    mov @r13+,2*1(r12)
    mov @r13+,2*2(r12)
    mov @r13+,2*3(r12)
#if (BC_CCFBH_TAG_LEN < 8)
    mov @r13+,2*4(r12)
#if (BC_CCFBH_TAG_LEN < 6)
    mov @r13+,2*5(r12)
#endif
#endif
    reta

RSEG CODE:CODE:REORDER:NOROOT(1)
ccfbh_tag_xor:
    xor @r13+,2*0(r12)
#if (BC_CCFBH_TAG_LEN > 2)
    xor @r13+,2*1(r12)
#if (BC_CCFBH_TAG_LEN > 4)
    xor @r13+,2*2(r12)
    xor @r13+,2*3(r12)
#endif
#endif
    reta

END
