/*
 * RELIC is an Efficient LIbrary for Cryptography
 * Copyright (C) 2007-2012 RELIC Authors
 *
 * This file is part of RELIC. RELIC is legal property of its developers,
 * whose names are not listed here. Please refer to the COPYRIGHT file
 * for contact information.
 *
 * RELIC is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * RELIC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file
 *
 * Implementation of the low-level authenticated cipher CCFBH functions.
 *
 * @version $Id$
 * @ingroup ac
 */

#include <msp430.h>

#include "relic_bc_conf.h"

#ifndef MSP430X
#define reta ret
#define RETA ret
#define calla call
#define CALLA call
#endif

//#define UNROLL

NAME relic_bc_hb2_asm
RSEG CODE
EVEN

PUBLIC ac_hb2_enc_lower
PUBLIC ac_hb2_dec_lower
PUBLIC ac_hb2_init_lower
EXTERN S12inv
EXTERN S34inv
EXTERN L0inv
EXTERN L1inv
EXTERN f0
EXTERN f1
EXTERN memcpy

/*============================================================================*/
/* Private definitions                                                        */
/*============================================================================*/

fm MACRO t1
    MOV.B   R12, t1
    SWPB    R12
    MOV.B   R12, R12

    //x0: t1
    //x1: R12

    //r12 = f1[x1]
    rla r12
    mov f1(r12), r12
    //r12 = f0[x0] ^ f1[x1]
    rla t1
    xor f0(t1), r12
 ENDM

finvm MACRO t1
    MOV.B   R12, t1
    SWPB    R12
    MOV.B   R12, R12

    //x0: t1
    //x1: R12

    //r12 = L1inv[x1]
    rla r12
    mov L1inv(r12), r12
    //r12 = L0inv[x0] ^ L1inv[x1]
    rla t1
    xor L0inv(t1), r12

    MOV.B   R12, t1
    MOV.B   S12inv(t1), t1
    SWPB    R12
    MOV.B   R12, R12
    MOV.B   S34inv(R12), R12
    SWPB    R12
    BIS.W   t1, R12
 ENDM

WD16m MACRO
    //Clobbers r11, r12
    fm r11
    xor r13, r12
    fm r11
    xor r14, r12
    fm r11
    xor r15, r12
    fm r11
 ENDM

WD16invm MACRO
    //Clobbers r11, r12
    finvm r11
    xor r15, r12
    finvm r11
    xor r14, r12
    finvm r11
    xor r13, r12
    finvm r11
 ENDM

WD16_:
    WD16m
    ret

#ifndef UNROLL
WD16inv:
    //push.w r11
    WD16invm
    //pop.w r11
    ret
#endif

/*============================================================================*/
/* Public definitions                                                         */
/*============================================================================*/

ac_hb2_init_lower:
#ifdef MSP430X
    PUSHM.W  #0x6, R11
#else
    push r11
    push r10
    push r9
    push r8
    push r7
    push r6
#endif
    //r12: ctx
    //r13: key
    //r15: iv

    //memcpy(ctx->K, k, sizeof(ctx->K));
    mov @r13+,2*0(r12)
    mov @r13+,2*1(r12)
    mov @r13+,2*2(r12)
    mov @r13+,2*3(r12)
    mov @r13+,2*4(r12)
    mov @r13+,2*5(r12)
    mov @r13+,2*6(r12)
    mov @r13+,2*7(r12)

    //memcpy(ctx->IV, iv, sizeof(ctx->IV));
    mov @r15+,r13
    mov r13,2*16(r12)
    mov r13,2*8(r12)
    mov r13,2*12(r12)
    mov @r15+,r13
    mov r13,2*17(r12)
    mov r13,2*9(r12)
    mov r13,2*13(r12)
    mov @r15+,r13
    mov r13,2*18(r12)
    mov r13,2*10(r12)
    mov r13,2*14(r12)
    mov @r15+,r13
    mov r13,2*19(r12)
    mov r13,2*11(r12)
    mov r13,2*15(r12)

    mov r12,r10

//  141     for (uint16_t i = 0; i < 4; i++) {
    MOV.W   #0x0, R8
    JMP     ??bc_a_init_iv_1
//  142         t1 = WD16((ctx->R[0] +  i) ^ k[0], k[1], k[2], k[3]);
??bc_a_init_iv_0:
    MOV.W   R8, R12
    ADD.W   0x10(R10), R12
    XOR.W   @R10+, R12
    MOV.W   @R10+, R13
    MOV.W   @R10+, R14
    MOV.W   @R10+, R15
    CALL   #WD16_
    MOV.W   R12, R9
//  143         t2 = WD16((ctx->R[1] + t1) ^ k[4], k[5], k[6], k[7]);
    ADD.W   0xa(R10), R12
    XOR.W   @R10+, R12
    MOV.W   @R10+, R13
    MOV.W   @R10+, R14
    MOV.W   @R10+, R15
    CALL   #WD16_
    MOV.W   R12, R6
//  144         t3 = WD16((ctx->R[2] + t2) ^ k[0], k[1], k[2], k[3]);
    sub #16,r10
    ADD.W   0x14(R10), R12
    XOR.W   @R10+, R12
    MOV.W   @R10+, R13
    MOV.W   @R10+, R14
    MOV.W   @R10+, R15
    CALL   #WD16_
    MOV.W   R12, R7
//  145         t4 = WD16((ctx->R[3] + t3) ^ k[4], k[5], k[6], k[7]);
    ADD.W   0xe(R10), R12
    XOR.W   @R10+, R12
    MOV.W   @R10+, R13
    MOV.W   @R10+, R14
    MOV.W   @R10+, R15
    CALL   #WD16_
//  146         t = (ctx->R[0] + t4);
    sub #16,r10
    ADD.W   0x10(R10), R12
//  147         ctx->R[0] = (t << 3) | (t >> 13);
    MOV.W   R12, R15
    SWPB    R15
    AND.B   #0xff, R15
    RRA.W   R15
    RRA.W   R15
    RRA.W   R15
    RRA.W   R15
    RRA.W   R15
#ifdef MSP430X
    RLAM.W  #0x3, R12
#else
    rla r12
    rla r12
    rla r12
#endif
    BIS.W   R15, R12
    MOV.W   R12, 0x10(R10)
//  148         t = (ctx->R[1] + t1);
    ADD.W   0x12(R10), R9
    MOV.W   R9, R12
//  149         ctx->R[1] = (t >> 1) | (t << 15);
    BIT.W   #0x1, R9
    RRC.W   R12
    MOV.W   R12, 0x12(R10)
//  150         t = (ctx->R[2] + t2);
    ADD.W   0x14(R10), R6
    MOV.W   R6, R12
//  151         ctx->R[2] = (t << 8) | (t >> 8);
    SWPB    R12
    MOV.W   R12, 0x14(R10)
//  152         t = (ctx->R[3] + t3);
    ADD.W   0x16(R10), R7
    MOV.W   R7, R12
//  153         ctx->R[3] = (t << 1) | (t >> 15);
    BIT.W   #0x8000, R7
    RLC.W   R12
    MOV.W   R12, 0x16(R10)
//  154         ctx->R[4] = ctx->R[4] ^ ctx->R[0];
    XOR.W   0x10(R10), 0x18(R10)
//  155         ctx->R[5] = ctx->R[5] ^ ctx->R[1];
    XOR.W   0x12(R10), 0x1a(R10)
//  156         ctx->R[6] = ctx->R[6] ^ ctx->R[2];
    XOR.W   0x14(R10), 0x1c(R10)
//  157         ctx->R[7] = ctx->R[7] ^ ctx->R[3];
    XOR.W   0x16(R10), 0x1e(R10)
//  158     }
    ADD.W   #0x1, R8
??bc_a_init_iv_1:
    CMP.W   #0x4, R8
    JNC     ??bc_a_init_iv_0
//  159 }
#ifdef MSP430X
    POPM.W  #0x6, R11
#else
    pop r6
    pop r7
    pop r8
    pop r9
    pop r10
    pop r11
#endif
    RETA

ac_hb2_enc_lower:
#ifdef MSP430X
    pushm.w #6,r11
#else
    push r11
    push r10
    push r9
    push r8
    push r7
    push r6
#endif
    //r12: ctx
    //r13: cipher
    //r14: msg
    mov r12,r6
    mov r13,r7
    mov @r14,r12
    //r6: ctx
    //r7: cipher
    //r12: msg

    //r12 = R[0] + p
    add 16(r6),r12
    //r12 ^= K[0]
    xor @r6+,r12
    //r13 = K[1]
    mov @r6+,r13
    //r14 = K[2]
    mov @r6+,r14
    //r15 = K[3]
    mov @r6+,r15
    //r12 = t1 = WD16(...)
#ifdef UNROLL
    WD16m
#else
    call #WD16_
#endif
    //r8 = t1
    mov r12,r8

    //r12 = r12 + R[1]
    add 10(r6),r12
    //r12 ^= K[4]
    xor @r6+,r12
    //r12 ^= R[4]
    xor 14(r6),r12
    //r13 = K[5]
    mov @r6+,r13
    //r13 ^= R[5]
    xor 14(r6),r13
    //r14 = K[6]
    mov @r6+,r14
    //r14 ^= R[6]
    xor 14(r6),r14
    //r15 = K[7]
    mov @r6+,r15
    //r15 ^= R[7]
    xor 14(r6),r15
    sub #16,r6
    //r12 = WD16(...)
#ifdef UNROLL
    WD16m
#else
    call #WD16_
#endif
    //r9 = t2
    mov r12,r9

    //r12 = r12 + R[2]
    add 20(r6),r12
    //r12 ^= K[0]
    xor @r6+,r12
    //r12 ^= R[4]
    xor 22(r6),r12
    //r13 = K[1]
    mov @r6+,r13
    //r13 ^= R[5]
    xor 22(r6),r13
    //r14 = K[2]
    mov @r6+,r14
    //r14 ^= R[6]
    xor 22(r6),r14
    //r15 = K[3]
    mov @r6+,r15
    //r15 ^= R[7]
    xor 22(r6),r15
    //r12 = WD16(...)
#ifdef UNROLL
    WD16m
#else
    call #WD16_
#endif
    //r10 = t3
    mov r12,r10

    //r12 = r12 + R[3]
    add 14(r6),r12
    //r12 ^= K[4]
    xor @r6+,r12
    //r13 = K[5]
    mov @r6+,r13
    //r14 = K[6]
    mov @r6+,r14
    //r15 = K[7]
    mov @r6+,r15
    //r12 = WD16(...)
#ifdef UNROLL
    WD16m
#else
    call #WD16_
#endif
    //r12 += R[0]
    add @r6,r12

    //cipher = r12
    mov r12,0(r7)

    //R[0] += t3
    add r10,0(r6)
    //R[1] += t1
    add r8,2(r6)
    //R[2] += t2
    add r9,4(r6)
    //R[3] += t1
    add r8,6(r6)
    //R[3] += R[0]
    add @r6,6(r6)
    //R[4] ^= R[0]
    xor @r6+,6(r6)
    //R[5] ^= R[1]
    xor @r6+,6(r6)
    //R[6] ^= R[2]
    xor @r6+,6(r6)
    //R[7] ^= R[3]
    xor @r6+,6(r6)

#ifdef MSP430X
    popm.w #6,r11
#else
    pop r6
    pop r7
    pop r8
    pop r9
    pop r10
    pop r11
#endif

    reta

ac_hb2_dec_lower:
#ifdef MSP430X
    pushm.w #6,r11
#else
    push r11
    push r10
    push r9
    push r8
    push r7
    push r6
#endif
    //r12: ctx
    //r13: msg
    //r14: cipher
    mov r12,r6
    mov r13,r7
    mov @r14,r12
    //r6: ctx
    //r7: msg
    //r12: cipher

    //r12 -= R[0]
    sub 16(r6),r12
    //r13 = K[5]
    mov 10(r6),r13
    //r14 = K[6]
    mov 12(r6),r14
    //r15 = K[7]
    mov 14(r6),r15
    //r12 = WD16(...)
#ifdef UNROLL
    WD16invm
#else
    call #WD16inv
#endif
    //r12 ^= K[4]
    xor 8(r6),r12
    //r12 = r12 - R[3]
    sub 22(r6),r12

    //r10 = t3
    mov r12,r10
    //r13 = K[1]
    mov 2(r6),r13
    //r13 ^= R[5]
    xor 26(r6),r13
    //r14 = K[2]
    mov 4(r6),r14
    //r14 ^= R[6]
    xor 28(r6),r14
    //r15 = K[3]
    mov 6(r6),r15
    //r15 ^= R[7]
    xor 30(r6),r15
    //r12 = WD16(...)
#ifdef UNROLL
    WD16invm
#else
    call #WD16inv
#endif
    //r12 ^= K[0]
    xor 0(r6),r12
    //r12 ^= R[4]
    xor 24(r6),r12
    //r12 = r12 - R[2]
    sub 20(r6),r12

    //r9 = t2
    mov r12,r9
    //r13 = K[5]
    mov 10(r6),r13
    //r13 ^= R[5]
    xor 26(r6),r13
    //r14 = K[6]
    mov 12(r6),r14
    //r14 ^= R[6]
    xor 28(r6),r14
    //r15 = K[7]
    mov 14(r6),r15
    //r15 ^= R[7]
    xor 30(r6),r15
    //r12 = WD16(...)
#ifdef UNROLL
    WD16invm
#else
    call #WD16inv
#endif
    //r12 ^= K[4]
    xor 8(r6),r12
    //r12 ^= R[4]
    xor 24(r6),r12
    //r12 = r12 - R[1]
    sub 18(r6),r12

    //r8 = t1
    mov r12,r8
    //r13 = K[1]
    mov 2(r6),r13
    //r14 = K[2]
    mov 4(r6),r14
    //r15 = K[3]
    mov 6(r6),r15
    //r12 = t1 = WD16(...)
#ifdef UNROLL
    WD16invm
#else
    call #WD16inv
#endif
    //r12 ^= K[0]
    xor 0(r6),r12
    //r12 = r12 - R[0]
    sub 16(r6),r12

    //msg = r12
    mov r12,0(r7)

    //R[0] += t3
    add r10,16(r6)
    //R[1] += t1
    add r8,18(r6)
    //R[2] += t2
    add r9,20(r6)
    //R[3] += t1
    add r8,22(r6)
    //R[3] += R[0]
    add 16(r6),22(r6)
    //R[4] ^= R[0]
    xor 16(r6),24(r6)
    //R[5] ^= R[1]
    xor 18(r6),26(r6)
    //R[6] ^= R[2]
    xor 20(r6),28(r6)
    //R[7] ^= R[3]
    xor 22(r6),30(r6)

#ifdef MSP430X
    popm.w #6,r11
#else
    pop r6
    pop r7
    pop r8
    pop r9
    pop r10
    pop r11
#endif

    reta

END
