/*
 * RELIC is an Efficient LIbrary for Cryptography
 * Copyright (C) 2007-2012 RELIC Authors
 *
 * This file is part of RELIC. RELIC is legal property of its developers,
 * whose names are not listed here. Please refer to the COPYRIGHT file
 * for contact information.
 *
 * RELIC is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * RELIC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file
 *
 * Implementation of the low-level authenticated cipher GCM functions.
 *
 * @version $Id$
 * @ingroup fb
 */

#include <string.h>

#include "relic_ac.h"
#include "relic_ac_low.h"
#include "relic_types.h"
#include "relic_util.h"

#ifdef AC_GCM_SGCM

/*============================================================================*/
/* Private definitions                                                        */
/*============================================================================*/

#define GCM_DIGS (AC_GCM_BLOCK_LEN / sizeof(dig_t))

//Defined in assembly.
void ac_gcm_muln_low(dig_t *c, dig_t *a, dig_t *b);
void ac_gcm_rdcn_low(dig_t *c, dig_t *a);

/*============================================================================*/
/* Public definitions                                                         */
/*============================================================================*/

void ac_gcm_tab_low(dig_t *c, unsigned char *a) {
	memcpy(c, a, AC_GCM_BLOCK_LEN);
}

void ac_gcm_mul_low(dig_t *c, dig_t *a, dig_t *b) {
	dig_t t[GCM_DIGS*2];

	ac_gcm_muln_low(t, a, b);
	ac_gcm_rdcn_low(c, t);
}

#endif /* #ifndef AC_GCM_SGCM */
