/*
 * RELIC is an Efficient LIbrary for Cryptography
 * Copyright (C) 2007-2012 RELIC Authors
 *
 * This file is part of RELIC. RELIC is legal property of its developers,
 * whose names are not listed here. Please refer to the COPYRIGHT file
 * for contact information.
 *
 * RELIC is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * RELIC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file
 *
 * Implementation of the low-level authenticated cipher CCFBH functions.
 *
 * @version $Id$
 * @ingroup ac
 */

#include "relic_bc_conf.h"

#ifdef AC_GCM_SGCM

#ifdef MSP430X
#include <msp430.h>
#else
/* For some reason these are not defined in asm */
#define MPY                0x0130  /* Multiply Unsigned/Operand 1 */
#define MPYS               0x0132  /* Multiply Signed/Operand 1 */
#define MAC                0x0134  /* Multiply Unsigned and Accumulate/Operand 1 */
#define MACS               0x0136  /* Multiply Signed and Accumulate/Operand 1 */
#define OP2                0x0138  /* Operand 2 */
#define RESLO              0x013A  /* Result Low Word */
#define RESHI              0x013C  /* Result High Word */
#define SUMEXT             0x013E  /* Sum Extend */
#endif

NAME relic_ac_sgcm_low_asm
RSEG CODE
EVEN

PUBLIC ac_gcm_muln_low
PUBLIC ac_gcm_rdcn_low
PUBLIC ac_gcm_inc_low

/*============================================================================*/
/* Public definitions                                                         */
/*============================================================================*/

RSEG CODE:CODE:REORDER:NOROOT(1)
ac_gcm_muln_low:
    mov r14,r15
    mov r12,r14
    mov r15,r12
    pushm.w #8,r11

#include "fp_mul32_128_comba.inc"

    popm.w #8,r11
    reta

RSEG CODE:CODE:REORDER:NOROOT(1)
ac_gcm_rdcn_low:

    pushm.w #8,r11

    /* compute q0.c, add lo(qo.c) to r0 */
    mov #0x30A3,&MPY

    add #2*8,r13

    /* 5 words */

    mov @r13+,&OP2
    mov &RESLO,r4
    mov &RESHI,r5
    clr r6
    clr r7

    mov @r13+,&OP2
    add &RESLO,r5
    addc &RESHI,r6
    adc r7
    clr r8

    mov @r13+,&OP2
    add &RESLO,r6
    addc &RESHI,r7
    adc r8
    clr r9

    mov @r13+,&OP2
    add &RESLO,r7
    addc &RESHI,r8
    adc r9
    clr r10

    mov @r13+,&OP2
    add &RESLO,r8
    addc &RESHI,r9
    adc r10
    clr r11

    sub r4,-2*13(r13)
    subc r5,-2*12(r13)
    subc r6,-2*11(r13)
    subc r7,-2*10(r13)
    subc r8,-2*9(r13)
    clr r15
    adc r15

    /* 3 words */

    mov @r13+,&OP2
    add &RESLO,r9
    addc &RESHI,r10
    adc r11
    clr r4

    mov @r13+,&OP2
    add &RESLO,r10
    addc &RESHI,r11
    adc r4

    mov @r13+,&OP2
    add &RESLO,r11
    addc &RESHI,r4

    clr r5
    clr r6
    rrc r15
    subc r9,-2*11(r13)
    subc r10,-2*10(r13)
    subc r11,-2*9(r13)
    subc r4,r5
    sbc r6

    jz skip3

    /* result is negative, add modulus to it */

    inc r6
    add #0x30A3,-2*15(r13)
    adc -2*14(r13)
    adc -2*13(r13)
    adc -2*12(r13)
    adc -2*11(r13)
    adc -2*10(r13)
    adc -2*9(r13)
    adc -2*8(r13)
    adc r5
    adc r6

skip3:

    sub #2*16,r13

    /* TODO: can be removed with renaming */
    mov r6,r7
    mov r5,r6

    /* q1 = high(q0.c) */
    /* compute q1.c, add lo(q1.c) to r1 */

    mov r6,&OP2
    mov &RESLO,r4
    mov &RESHI,r5
    clr r6

    mov r7,&OP2
    add &RESLO,r5
    addc &RESHI,r6

    mov @r13+,r7
    sub r4,r7
    mov r7,2*0(r12)
    mov @r13+,r7
    subc r5,r7
    mov r7,2*1(r12)
    mov @r13+,r7
    subc r6,r7
    mov r7,2*2(r12)
    mov @r13+,r7
    sbc r7
    mov r7,2*3(r12)
    mov @r13+,r7
    sbc r7
    mov r7,2*4(r12)
    mov @r13+,r7
    sbc r7
    mov r7,2*5(r12)
    mov @r13+,r7
    sbc r7
    mov r7,2*6(r12)
    mov @r13+,r7
    sbc r7
    mov r7,2*7(r12)
    clr r7
    sbc r7

    jz skip2

    /* result is negative, add modulus to it */

    inc r7
    add #0x30A3,-2*8(r13)
    adc -2*7(r13)
    adc -2*6(r13)
    adc -2*5(r13)
    adc -2*4(r13)
    adc -2*3(r13)
    adc -2*2(r13)
    adc -2*1(r13)

skip2:
    popm.w #8,r11
    reta

#endif

END

