/*
 * RELIC is an Efficient LIbrary for Cryptography
 * Copyright (C) 2007-2012 RELIC Authors
 *
 * This file is part of RELIC. RELIC is legal property of its developers,
 * whose names are not listed here. Please refer to the COPYRIGHT file
 * for contact information.
 *
 * RELIC is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * RELIC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file
 *
 * Implementation of the low-level authenticated cipher CCFBH functions.
 *
 * @version $Id$
 * @ingroup ac
 */

#include <msp430.h>

#include "relic_bc_conf.h"

#ifndef MSP430X
#define reta ret
#define RETA ret
#define calla call
#define CALLA call
#endif

NAME relic_bc_aes_low_asm
RSEG CODE
EVEN

EXTERN mule9
EXTERN mulbe
EXTERN muldb
EXTERN mul9d
EXTERN isbox

#ifdef BC_SINGL

PUBLIC bc_aes128_enc_start
PUBLIC bc_aes128_dec_start

#ifdef BC_SPLIT

PUBLIC bc_aes128_enc_end
PUBLIC bc_aes128_enc_end

RSEG CODE:CODE:REORDER:NOROOT(1)
bc_aes128_enc_start:
bc_aes128_dec_start:
    mov @r14+,&AESADIN
    mov @r14+,&AESADIN
    mov @r14+,&AESADIN
    mov @r14+,&AESADIN
    mov @r14+,&AESADIN
    mov @r14+,&AESADIN
    mov @r14+,&AESADIN
    mov @r14+,&AESADIN
    reta

RSEG CODE:CODE:REORDER:NOROOT(1)
bc_aes128_enc_end:
bc_aes128_dec_end:
back:
    bit.w #AESBUSY,&AESASTAT
    jc back

    mov &AESADOUT,2*0(r13)
    mov &AESADOUT,2*1(r13)
    mov &AESADOUT,2*2(r13)
    mov &AESADOUT,2*3(r13)
    mov &AESADOUT,2*4(r13)
    mov &AESADOUT,2*5(r13)
    mov &AESADOUT,2*6(r13)
    mov &AESADOUT,2*7(r13)

    reta

#else

RSEG CODE:CODE:REORDER:NOROOT(1)
bc_aes128_enc_start:
bc_aes128_dec_start:
	mov @r14+,&AESADIN
	mov @r14+,&AESADIN
	mov @r14+,&AESADIN
	mov @r14+,&AESADIN
	mov @r14+,&AESADIN
	mov @r14+,&AESADIN
	mov @r14+,&AESADIN
	mov @r14+,&AESADIN

back:
	bit.w #AESBUSY,&AESASTAT
	jc back

	mov &AESADOUT,2*0(r13)
	mov &AESADOUT,2*1(r13)
	mov &AESADOUT,2*2(r13)
	mov &AESADOUT,2*3(r13)
	mov &AESADOUT,2*4(r13)
	mov &AESADOUT,2*5(r13)
	mov &AESADOUT,2*6(r13)
	mov &AESADOUT,2*7(r13)

	reta

#endif

#endif

END

