from collections import deque
from fb_mul_macros_128 import *

LTR = 0
RTL = 1

def mul(printer, ndigs, nbits_dig, nbits_win, blocks, order, add=1):
    nsubdigs = nbits_dig // nbits_win
    nbdigs = ndigs + add
    nblockdigs = nbdigs // blocks
    if order == RTL:
        for dig in xrange(ndigs):
            for subdig in xrange(nsubdigs):
                printer.step(subdig, dig)
    else:
        for subdig in xrange(nsubdigs - 1, -1, -1):
            for startbdig in xrange(0, nbdigs, nblockdigs):
                for adig in xrange(ndigs):
                    for bdig in xrange(startbdig, startbdig + nblockdigs):
                        printer.step(subdig, adig, bdig, bdig - startbdig)
                printer.end_block(startbdig, subdig)
            printer.end_subdig(subdig)

class Context(object):
    def __init__(self):
        self.a = None
        self.c = None
        self.t = None

class Generator():
    def __init__(self, ndigs, nbits_dig, nbits_win, blocks, add=1):
        self.ndigs = ndigs
        self.nbdigs = ndigs + add
        self.nbits_dig = nbits_dig
        self.nbits_win = nbits_win
        self.nsubdigs = nbits_dig // nbits_win
        self.nblockdigs = self.nbdigs // blocks
        self.lines = []
        self.code = []
        self.c = Context()
        self.c.a = 'r14'
        self.c.c = 'r15'
        self.c.acc = deque('r5 r6 r7 r8 r9 r10 r11 r12'.split())
        #Which digits of c have been written before
        self.written = [False] * self.ndigs * 2
        self.INDEX_MULT = INDEX_MULT

#        self.code.append(BENCH_RESET)

    def step(self, subdig, adig, bdig, brdig):
        if brdig == 0:
            idx_read = globals()['INDEX_READ_%d' % subdig]
            idx_read = idx_read.format(**self.c.__dict__)
            self.code.append(idx_read)
            if subdig % 2 == 1:
                idx_mult = self.INDEX_MULT_ALT.format(offset=bdig*2)
            else:
                idx_mult = self.INDEX_MULT.format(offset=bdig*2)
            self.code.append(idx_mult)
            col = COL_I_MOV if adig == 0 or brdig == self.nblockdigs - 1 else COL_I_XOR
            col = col.format(acci=self.c.acc[0])
            self.code.append(col)
            cdig = adig + bdig
            if self.written[cdig]:
                col_write = COL_WRITE_XOR
            else:
                col_write = COL_WRITE_MOV
                self.written[cdig] = True
            col_write = col_write.format(acci=self.c.acc[0], offset=(adig+bdig)*2, **self.c.__dict__)
            self.code.append(col_write)
            self.c.acc.rotate(-1)
        else:
            col = COL_I_MOV if adig == 0 or brdig == self.nblockdigs - 1 else COL_I_XOR
            col = col.format(acci=self.c.acc[brdig-1])
            self.code.append(col)

    def end_block(self, startbdig, subdig):
        start = startbdig + self.ndigs
        for i, cdig in enumerate(xrange(start, start + self.nblockdigs - 1)):
            if self.written[cdig]:
                col_write = COL_WRITE_XOR
            else:
                col_write = COL_WRITE_MOV
                self.written[cdig] = True
            col_write = col_write.format(acci=self.c.acc[i], offset=(cdig)*2, **self.c.__dict__)
            self.code.append(col_write)
        sub = SUB.format(offset=self.ndigs*2, r=self.c.a)
        self.code.append(sub)

    def end_subdig(self, subdig):
        if subdig == 0:
            return
#        self.code.append(BENCH_START)
        for _ in xrange(self.nsubdigs):
            shift = SHIFT_0.format(**self.c.__dict__)
            self.code.append(shift)
            for _ in xrange(self.ndigs * 2 - 1):
                shift = SHIFT.format(**self.c.__dict__)
                self.code.append(shift)
            sub = SUB.format(offset=self.ndigs*2*2, r=self.c.c)
            self.code.append(sub)
#        self.code.append(BENCH_STOP)

    def gen_code(self):
#        self.code.append(BENCH_PRINT)
        return ''.join(self.code)

class GeneratorFasterShift(Generator):
    def __init__(self, ndigs, nbits_dig, nbits_win, blocks, add=1):
        Generator.__init__(self, ndigs, nbits_dig, nbits_win, blocks, add)

    def end_subdig(self, subdig):
        if subdig == 0:
            return
#        self.code.append(BENCH_START)
        self.code.append(SHIFT_FAST_START)
        for _ in xrange(self.nsubdigs - 1):
            shift = SHIFT_FAST.format(**self.c.__dict__)
            self.code.append(shift)
            self.code.append(SHIFT_FAST_CARRY)
        shift = SHIFT_FAST_LAST_7.format(**self.c.__dict__)
        self.code.append(shift)
#        self.code.append(BENCH_STOP)

class GeneratorKaratsuba(GeneratorFasterShift):
    def __init__(self, ndigs, nbits_dig, nbits_win, blocks):
        GeneratorFasterShift.__init__(self, ndigs, nbits_dig, nbits_win, blocks)
        self.INDEX_MULT = INDEX_MULT_KARATSUBA
        self.c.acc.appendleft('r5')

    def end_subdig(self, subdig):
        if subdig == 0:
            return
#        self.code.append(BENCH_START)
        self.code.append(SHIFT_FAST_START)
        for _ in xrange(2):
            shift = SHIFT_FAST.format(**self.c.__dict__)
            self.code.append(shift)
            self.code.append(SHIFT_FAST_CARRY)
        self.code.append(SUB.format(offset=2*18, r=self.c.c))
#        self.code.append(BENCH_STOP)


class GeneratorFasterKaratsuba(GeneratorFasterShift):
    def __init__(self, ndigs, nbits_dig, nbits_win, blocks, add):
        GeneratorFasterShift.__init__(self, ndigs, nbits_dig, nbits_win, blocks, add)
        self.INDEX_MULT = INDEX_MULT_KARATSUBA
        self.INDEX_MULT_ALT = INDEX_MULT_KARATSUBA_ALT

    def end_block(self, startbdig, subdig):
        start = startbdig + self.ndigs
        for i, cdig in enumerate(xrange(start, start + self.nblockdigs - 1)):
            self.written[cdig] = True
            col_write = COL_WRITE_XOR
            col_write = col_write.format(acci=self.c.acc[i], offset=(cdig)*2, **self.c.__dict__)
            #Since end_subdig writes the columns, we only need for
            #the 0th subdig which does not need shift
            if subdig == 0:
                self.code.append(col_write)
        sub = SUB.format(offset=self.ndigs*2, r=self.c.a)
        self.code.append(sub)

    def end_subdig(self, subdig):
        if subdig == 0:
            return
        if subdig % 2 == 1:
            if subdig == self.nsubdigs - 1:
                code = JUST_COPY.format(**self.c.__dict__)
            else:
                code = JUST_XOR.format(**self.c.__dict__)
            self.code.append(code)
            return
#        self.code.append(BENCH_START)
        shift_xors = '' if subdig == self.nsubdigs - 1 else SHIFT_FAST_KARATSUBA_XORS.format(**self.c.__dict__)
        shift = SHIFT_FAST_KARATSUBA.format(xors=shift_xors, **self.c.__dict__)
        self.code.append(shift)
        self.code.append(SHIFT_FAST_START)
        shift = SHIFT_FAST.format(**self.c.__dict__)
        self.code.append(shift)
        self.code.append(SUB.format(offset=2*8, r=self.c.c))
#        self.code.append(BENCH_STOP)

class Printer():
    def __init__(self, ndigs, nbits_dig, nbits_win, blocks, add=1):
        self.ndigs = ndigs
        self.nbdigs = ndigs + add
        self.nbits_dig = nbits_dig
        self.nbits_win = nbits_win
        self.nsubdigs = nbits_dig // nbits_win
        self.nblockdigs = self.nbdigs // blocks
        self.lines = []

    def step(self, subdig, adig, bdig, brdig):
        if bdig == 0:
            label = "a{0},{1}".format(adig, subdig)
            line = "|" + "|".join("{0:{2}>{1}}".format('', self.nsubdigs - 1, '*' if i < self.nblockdigs else '.')
                                  for i in xrange(self.nbdigs))
            line += "|{0:>5}".format(label)
            line = line + " " * (adig * self.nsubdigs)
            self.lines.append(line)

    def end_block(self, startbdig, subdig):
        pass

    def end_subdig(self, subdig):
        pass

    def printcalc(self):
        mlen = max(len(line) for line in self.lines)
        for line in self.lines:
            print "{0:>{1}}".format(line, mlen)



ndigs = 8
nbits_dig = 16
nbits_win = 4

blocks = 1
add = 1

order = LTR

p = Printer(ndigs, nbits_dig, nbits_win, blocks, add)
g = GeneratorFasterKaratsuba(ndigs, nbits_dig, nbits_win, blocks, add)

mul(g, ndigs, nbits_dig, nbits_win, blocks, order, add)
#print g.gen_code()

mul(p, ndigs, nbits_dig, nbits_win, blocks, order, add)
p.printcalc()

with open('fb_mul_128_ld.inc', 'w') as f:
    f.write(g.gen_code())
