from fp_mul32_macros import *

def mult(param, printer):
    printer.start()

    i = n = 0
    while i < param:
        k = 0
        m = i
        c = 0
        while k <= i:
            printer.step(k, m, c, 0)
            k += 1
            m -= 1
            c += 1
        printer.column_end(n, c)
        printer.dec(i+1, n)
        i += 1
        n += 1

    printer.change_stage()

    i = 0
    while i < param - 1:
        k = i + 1
        m = param - 1
        c = 0
        while k <= param - 1:
            printer.step(k, m, c, 1)
            k += 1
            m -= 1
            c += 1
        printer.column_end(n, c)
        printer.dec(param - (i+1) - 1, n)
        i += 1
        n += 1

    printer.end(n)


class MultContext(object):
    def __init__(self):
        self.a = None
        self.b = None
        self.c = None
        self.carry = None
        self.sumext = None


class Printer(object):
    def __init__(self, param, simple_cache, alt):
        self.param = param
        self.simple_cache = simple_cache
        self.alt = alt
        self.lines = []
        self.tab = ''
        self.code = []
        self.c = MultContext()
        self.c.a = "r15"
        self.c.b = "r14"
        self.c.c = "r13"
        self.c.sumext = "r12"
        self.c.carry = "r11"
        if alt:
            self.cache = ["r4", "r5", "r6", "r7", "r8"]
        else:
            self.cache = [None, None, "r4", "r5", "r6", "r7", "r8", "r9"]
        self.cache_size = len(self.cache) // 2 - 1
        self.cache += [None, None] * (self.param - len(self.cache) // 2)
        self.cached = [False] * self.param
        self.numstep = 0
        self.col = 0
        self.cache_hits = 0
        self.cache_misses = 0
        self.non_cached = 0

    def start(self):
        t = START_ALT if self.alt else START
        self.code.append(t.format(c=self.c))

    def step(self, i, j, numsteps, stage):
        if self.numstep == 0:
            code = STEP_FIRST
        elif self.numstep == 1:
            code = STEP_SECOND
        elif numsteps == 0:
            code = STEP_THIRD_MORE_1
        else:
            code = STEP_THIRD_MORE
        self.numstep += 1
        reg = self.cache[2*j]
        if reg is not None:
            cache_code = ''
            if not self.cached[j]:
                cache_code = STEP_SAVECACHE.format(boffset=2*j,
                                                   cache1=self.cache[2*j],
                                                   cache2=self.cache[2*j+1],
                                                   c=self.c)
                self.cache_misses += 1
                hit = 0
            else:
                self.cache_hits += 1
                hit = 1
            self.cached[j] = True
            cache_code += STEP_CACHED.format(cache1=self.cache[2*j],
                                             cache2=self.cache[2*j+1],
                                             c=self.c)
        else:
            cache_code = STEP_NONCACHED.format(boffset=2*j, c=self.c)
            self.non_cached += 1
            hit = 2
        self.code.append(code.format(cache_code=cache_code, c=self.c, boffset=2*j))
        self.lines.append("{0:>3}.{1:>3}|{2}{3:8}".format('a{0}'.format(i), 'b{0}'.format(j), self.tab, 'STEP' + 'MHN'[hit]))

    def change_stage(self):
        self.lines.append("{0:8}".format('STAGE'))

    def column_end(self, n, numsteps):
        self.tab += '   |'
        self.col += 1
        if not self.simple_cache:
            ncol = self.col - self.param - 1
            rcol = self.param - ncol - 1
            if ncol >= 0 and ncol < self.cache_size and self.cache[2*rcol] is None:
                self.lines.append(str(ncol))
                ncol += 1
                reg1 = self.cache[2*ncol]
                reg2 = self.cache[2*ncol+1]
                self.cache[2*ncol] = None
                self.cache[2*ncol+1] = None
                self.cached[ncol] = False
                self.cache[2*rcol] = reg1
                self.cache[2*rcol+1] = reg2
        if n == 0:
            code = COLUMN_END_FIRST_ALT if self.alt else COLUMN_END_FIRST
        else:
            code = COLUMN_END_SECOND_MORE_ALT if self.alt else COLUMN_END_SECOND_MORE
        self.code.append(code.format(c=self.c, coffset=2*n))

    def dec(self, n, idx):
        self.lines.append("{0:8}".format('DEC {0} {1}'.format(n, idx)))
        if n == self.param:
            #hack because the last dec in the first stage is
            #actually the first dec in the second stage
            n -= 1
        if n != 0:
            self.code.append(DEC.format(c=self.c, offset=2*n))

    def end(self, n):
        self.code.append(DEC.format(c=self.c, offset=2*self.param))
        t = END_ALT if self.alt else END
        self.code.append(t.format(c=self.c, coffset=2*n))

    def printcalc(self):
        for line in self.lines:
            print "{0:>{1}}".format(line, 33*4+8)
        print 'cache hits:', self.cache_hits
        print 'cache misses:', self.cache_misses
        print 'cache ignored:', self.non_cached

    def gen_code(self):
        return ''.join(self.code)

PARAM = 4
p = Printer(PARAM, False, False)

mult(PARAM, p)
p.printcalc()

with open('fp_mul32_128_comba.inc', 'w') as f:
    f.write(p.gen_code())
