#if 1
/*
 * RELIC is an Efficient LIbrary for Cryptography
 * Copyright (C) 2007-2012 RELIC Authors
 *
 * This file is part of RELIC. RELIC is legal property of its developers,
 * whose names are not listed here. Please refer to the COPYRIGHT file
 * for contact information.
 *
 * RELIC is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * RELIC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file
 *
 * Tests for authenticated ciphers.
 *
 * @version $Id$
 * @ingroup test
 */

#ifdef __ICC430__
#include <msp430.h>
#endif

#include <stdio.h>

#include "relic.h"
#include "relic_test.h"

static int ccm(void) {
	int code = STS_ERR;
	unsigned char tag[AC_CCM_TAG_LEN];
	ac_ccm_ctx_t ctx;
	bc_t t;

#if AC_CCM_TAG_LEN == 8
	TEST_BEGIN("CCM passes test vector 3") {
		unsigned char key[] = {0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f};
		unsigned char iv[] = {0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b};
		unsigned char data[] = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x10, 0x11, 0x12, 0x13};
		unsigned char msg[] = {0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37};
		unsigned char cip[] = {0xe3, 0xb2, 0x01, 0xa9, 0xf5, 0xb7, 0x1a, 0x7a, 0x9b, 0x1c, 0xea, 0xec, 0xcd, 0x97, 0xe7, 0x0b, 0x61, 0x76, 0xaa, 0xd9, 0xa4, 0x42, 0x8a, 0xa5};
		unsigned char tag_ref[] = {0x48, 0x43, 0x92, 0xfb, 0xc1, 0xb0, 0x99, 0x51};

		ac_ccm_key(ctx, key, sizeof(key));
		ac_ccm_init(ctx, key, sizeof(key), iv, sizeof(iv), sizeof(msg), sizeof(data));
		ac_ccm_data(ctx, data, AC_CCM_BLOCK_LEN);
		ac_ccm_data(ctx, data + AC_CCM_BLOCK_LEN, sizeof(data) - AC_CCM_BLOCK_LEN);
		ac_ccm_data(ctx, NULL, 0);
		ac_ccm_enc(ctx, t, msg, AC_CCM_BLOCK_LEN);
		TEST_ASSERT(memcmp(t, cip, AC_CCM_BLOCK_LEN) == 0, end);
		ac_ccm_enc(ctx, t, msg + AC_CCM_BLOCK_LEN, AC_CCM_BLOCK_LEN / 2);
		TEST_ASSERT(memcmp(t, cip + AC_CCM_BLOCK_LEN, AC_CCM_BLOCK_LEN / 2) == 0, end);
		ac_ccm_tag(ctx, tag, sizeof(tag));
		TEST_ASSERT(memcmp(tag, tag_ref, sizeof(tag)) == 0, end);
	}
	TEST_END;
#endif
	code = STS_OK;
end:
	return code;
}

void dump(void *p, int len) {
	unsigned char *a = p;
	for (int i = 0; i < len; i++) {
		printf("%02X", a[i]);
	}
	puts("");
}
static int gcm(void) {
	int code = STS_ERR;
	unsigned char key[BC_KEY_LEN];
	unsigned char iv[AC_GCM_IV_LEN];
	align unsigned char msg[AC_GCM_BLOCK_LEN];
	align unsigned char cipher[AC_GCM_BLOCK_LEN];
	unsigned char tag[AC_GCM_TAG_LEN];
	ac_gcm_ctx_t ctx;


#if BC_METHD == AES128 && !defined(AC_GCM_SGCM)
//	align unsigned char buf[AC_GCM_BLOCK_LEN * 2];
//	memset(msg, 0, sizeof(msg));
//	memset(cipher, 0, sizeof(cipher));
//	msg[8] = 1;
//	cipher[8] = 1;
//	dump(msg, sizeof(msg));
//	dump(cipher, sizeof(cipher));
//	ac_gcm_mul_low(buf, msg, cipher);
//	dump(buf, sizeof(buf));

	TEST_ONCE("GCM passes test vector 13") {
		unsigned char tag_ref[] = { 0x58, 0xe2, 0xfc, 0xce, 0xfa, 0x7e, 0x30, 0x61, 0x36, 0x7f, 0x1d, 0x57, 0xa4, 0xe7, 0x45, 0x5a };
		memset(key, 0, sizeof(key));
		memset(iv, 0, sizeof(iv));
		ac_gcm_key(ctx, key, sizeof(key));
		ac_gcm_init(ctx, key, sizeof(key), iv, sizeof(iv), 0, 0);
		ac_gcm_tag(ctx, tag, sizeof(tag));
		TEST_ASSERT(memcmp(tag, tag_ref, sizeof(tag)) == 0, end);
	}
	TEST_END;

	TEST_ONCE("GCM passes test vector 14") {
		align unsigned char cipher_ref[] = { 0x03, 0x88, 0xda, 0xce, 0x60, 0xb6, 0xa3, 0x92, 0xf3, 0x28, 0xc2, 0xb9, 0x71, 0xb2, 0xfe, 0x78 };
		unsigned char tag_ref[] = { 0xab, 0x6e, 0x47, 0xd4, 0x2c, 0xec, 0x13, 0xbd, 0xf5, 0x3a, 0x67, 0xb2, 0x12, 0x57, 0xbd, 0xdf };
		memset(key, 0, sizeof(key));
		memset(iv, 0, sizeof(iv));
		memset(msg, 0, sizeof(msg));
		ac_gcm_key(ctx, key, sizeof(key));
		ac_gcm_init(ctx, key, sizeof(key), iv, sizeof(iv), 0, 0);
		ac_gcm_enc(ctx, cipher, msg, sizeof(msg));
		ac_gcm_tag(ctx, tag, sizeof(tag));
		TEST_ASSERT(memcmp(cipher, cipher_ref, sizeof(cipher)) == 0, end);
		TEST_ASSERT(memcmp(tag, tag_ref, sizeof(tag)) == 0, end);
	}
	TEST_END;
#endif

#if 0
//#if BC_METHD == AES256
	TEST_ONCE("GCM passes test vector 16") {
		align unsigned char cipher1[AC_CCM_BLOCK_LEN];
		align unsigned char cipher2[AC_CCM_BLOCK_LEN];
		align unsigned char cipher3[AC_CCM_BLOCK_LEN];
		align unsigned char cipher4[AC_CCM_BLOCK_LEN];
		unsigned char key[] = { 0xfe, 0xff, 0xe9, 0x92, 0x86, 0x65, 0x73, 0x1c, 0x6d, 0x6a, 0x8f, 0x94, 0x67, 0x30, 0x83, 0x08,
									   0xfe, 0xff, 0xe9, 0x92, 0x86, 0x65, 0x73, 0x1c, 0x6d, 0x6a, 0x8f, 0x94, 0x67, 0x30, 0x83, 0x08 };
		align unsigned char msg1[] = { 0xd9, 0x31, 0x32, 0x25, 0xf8, 0x84, 0x06, 0xe5, 0xa5, 0x59, 0x09, 0xc5, 0xaf, 0xf5, 0x26, 0x9a };
		align unsigned char msg2[] = { 0x86, 0xa7, 0xa9, 0x53, 0x15, 0x34, 0xf7, 0xda, 0x2e, 0x4c, 0x30, 0x3d, 0x8a, 0x31, 0x8a, 0x72 };
		align unsigned char msg3[] = { 0x1c, 0x3c, 0x0c, 0x95, 0x95, 0x68, 0x09, 0x53, 0x2f, 0xcf, 0x0e, 0x24, 0x49, 0xa6, 0xb5, 0x25 };
		align unsigned char msg4[] = { 0xb1, 0x6a, 0xed, 0xf5, 0xaa, 0x0d, 0xe6, 0x57, 0xba, 0x63, 0x7b, 0x39 };
		align unsigned char msga1[] = { 0xfe, 0xed, 0xfa, 0xce, 0xde, 0xad, 0xbe, 0xef, 0xfe, 0xed, 0xfa, 0xce, 0xde, 0xad, 0xbe, 0xef };
		align unsigned char msga2[] = { 0xab, 0xad, 0xda, 0xd2 };
		unsigned char iv[] = { 0xca, 0xfe, 0xba, 0xbe, 0xfa, 0xce, 0xdb, 0xad, 0xde, 0xca, 0xf8, 0x88 };
		align unsigned char cipher1_ref[] = { 0x52, 0x2d, 0xc1, 0xf0, 0x99, 0x56, 0x7d, 0x07, 0xf4, 0x7f, 0x37, 0xa3, 0x2a, 0x84, 0x42, 0x7d };
		align unsigned char cipher2_ref[] = { 0x64, 0x3a, 0x8c, 0xdc, 0xbf, 0xe5, 0xc0, 0xc9, 0x75, 0x98, 0xa2, 0xbd, 0x25, 0x55, 0xd1, 0xaa };
		align unsigned char cipher3_ref[] = { 0x8c, 0xb0, 0x8e, 0x48, 0x59, 0x0d, 0xbb, 0x3d, 0xa7, 0xb0, 0x8b, 0x10, 0x56, 0x82, 0x88, 0x38 };
		align unsigned char cipher4_ref[] = { 0xc5, 0xf6, 0x1e, 0x63, 0x93, 0xba, 0x7a, 0x0a, 0xbc, 0xc9, 0xf6, 0x62 };
		unsigned char tag_ref[] = { 0x76, 0xfc, 0x6e, 0xce, 0x0f, 0x4e, 0x17, 0x68, 0xcd, 0xdf, 0x88, 0x53, 0xbb, 0x2d, 0x55, 0x1b };
		ac_gcm_key(ctx, key, sizeof(key));
		ac_gcm_init(ctx, key, sizeof(key), iv, sizeof(iv), 0, 0);
		ac_gcm_data(ctx, msga1, sizeof(msga1));
		ac_gcm_data(ctx, msga2, sizeof(msga2));
		ac_gcm_enc(ctx, cipher1, msg1, sizeof(msg1));
		ac_gcm_enc(ctx, cipher2, msg2, sizeof(msg2));
		ac_gcm_enc(ctx, cipher3, msg3, sizeof(msg3));
		ac_gcm_enc(ctx, cipher4, msg4, sizeof(msg4));
		ac_gcm_tag(ctx, tag, sizeof(tag));
		TEST_ASSERT(memcmp(cipher1, cipher1_ref, sizeof(cipher1_ref)) == 0, end);
		TEST_ASSERT(memcmp(cipher2, cipher2_ref, sizeof(cipher2_ref)) == 0, end);
		TEST_ASSERT(memcmp(cipher3, cipher3_ref, sizeof(cipher3_ref)) == 0, end);
		TEST_ASSERT(memcmp(cipher4, cipher4_ref, sizeof(cipher4_ref)) == 0, end);
		TEST_ASSERT(memcmp(tag, tag_ref, sizeof(tag_ref)) == 0, end);
	}
	TEST_END;
#endif
	code = STS_OK;
end:
	return code;
}

static int ocb3(void) {
	int code = STS_ERR;
	ac_ocb3_ctx_t ctx;

	TEST_ONCE("OCB3 passes test vector") {
		align unsigned char msg[2 * AC_OCB3_BLOCK_LEN];
		align unsigned char data[2 * AC_OCB3_BLOCK_LEN];
		align unsigned char cipher[2 * AC_OCB3_BLOCK_LEN];
		unsigned char tag[AC_OCB3_TAG_LEN];
		unsigned char iv[] = {'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l'};
		unsigned char key[] = {'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'};
		align unsigned char cip1[] = {0xb7, 0x04, 0x58, 0x6a, 0xc2, 0x77, 0xf0, 0xca, 0x0c, 0x23, 0x40, 0x2c, 0x23, 0x43, 0x8e, 0xa9};
		align unsigned char cip2[] = {0x07, 0x86, 0x58, 0xcf, 0x29, 0x0b, 0xef, 0x8e, 0x60, 0x78, 0x68, 0x01, 0x10, 0x33, 0xd6, 0xbf};
		unsigned char tag_ref[] = {0xc4, 0x11, 0xe5, 0x03, 0xce, 0x24, 0x58, 0xe5, 0x7e, 0xde, 0x66, 0x93, 0xed, 0xd0, 0x6b, 0xce};

		memset(msg, 0, sizeof(msg));
		memset(data, 0, sizeof(data));
		msg[0] = 1;
		data[0] = 2;

		ac_ocb3_key(ctx, key, sizeof(key));
		ac_ocb3_init(ctx, key, sizeof(key), iv, sizeof(iv), sizeof(msg), sizeof(data));
		ac_ocb3_data(ctx, data, AC_OCB3_BLOCK_LEN);
		ac_ocb3_data(ctx, data + AC_OCB3_BLOCK_LEN, AC_OCB3_BLOCK_LEN);
		ac_ocb3_enc(ctx, cipher, msg, AC_OCB3_BLOCK_LEN);
		TEST_ASSERT(memcmp(cipher, cip1, AC_OCB3_BLOCK_LEN) == 0, end);
		ac_ocb3_enc(ctx, cipher + AC_OCB3_BLOCK_LEN, msg + AC_OCB3_BLOCK_LEN, AC_OCB3_BLOCK_LEN);
		TEST_ASSERT(memcmp(cipher + AC_OCB3_BLOCK_LEN, cip2, AC_OCB3_BLOCK_LEN) == 0, end);
		ac_ocb3_tag(ctx, tag, sizeof(tag));
		TEST_ASSERT(memcmp(tag, tag_ref, sizeof(tag)) == 0, end);
	}
	TEST_END;
	code = STS_OK;
end:
	return code;
}

static int hb2(void) {
	int code = STS_ERR;
	unsigned char tag[AC_HB2_TAG_LEN];
	align unsigned char t1[16];
	align unsigned char t2[16];
	ac_hb2_ctx_t ctx;

	TEST_ONCE("HB2 passes test vector 1") {
		unsigned char key[AC_HB2_KEY_LEN] = {0};
		unsigned char iv[AC_HB2_IV_LEN] = {0};
		align unsigned char msg[16] = {0};
		align unsigned char ref_cipher[] = {0xc4, 0xef, 0x87, 0xa8, 0x4f, 0x05, 0xa9, 0x91, 0x57, 0x46, 0x44, 0x81, 0x6e, 0x25, 0x3a, 0xcf};
		unsigned char ref_tag[] = {0xba, 0xed, 0x40, 0xf0, 0x67, 0xb0, 0xe1, 0x3c, 0x76, 0xf3, 0x59, 0x41, 0xa2, 0xb2, 0xd1, 0x35};

		ac_hb2_key(ctx, key, sizeof(key));
		ac_hb2_init(ctx, key, sizeof(key), iv, sizeof(iv), 0, 0);
		for (int j = 0; j < sizeof(msg); j += AC_HB2_BLOCK_LEN) {
			ac_hb2_enc(ctx, t2 + j, msg + j, AC_HB2_BLOCK_LEN);
		}
		ac_hb2_tag(ctx, tag, sizeof(tag));
		TEST_ASSERT(memcmp(t2, ref_cipher, sizeof(ref_cipher)) == 0, end);
		TEST_ASSERT(memcmp(tag, ref_tag, sizeof(ref_tag)) == 0, end);

		ac_hb2_key(ctx, key, sizeof(key));
		ac_hb2_init(ctx, key, sizeof(key), iv, sizeof(iv), 0, 0);
		for (int j = 0; j < sizeof(msg); j += AC_HB2_BLOCK_LEN) {
			ac_hb2_dec(ctx, t1 + j, t2 + j, AC_HB2_BLOCK_LEN);
		}
		TEST_ASSERT(memcmp(msg, t1, sizeof(t1)) == 0, end);
		TEST_ASSERT(ac_hb2_check(ctx, tag, sizeof(tag)) == 1, end);
	}
	TEST_END;

	TEST_ONCE("HB2 passes test vector 2") {
		unsigned char key[] = {0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef, 0xfe, 0xdc, 0xba, 0x98, 0x76, 0x54, 0x32, 0x10};
		unsigned char iv[] = {0x12, 0x34, 0x56, 0x78, 0x9a, 0xbc, 0xde, 0xf0};
		align unsigned char msg[] = {0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff};
		align unsigned char ref_cipher[] = {0x5b, 0xd1, 0xf8, 0xad, 0x23, 0x14, 0x20, 0xf4, 0xba, 0xb1, 0x54, 0xc2, 0x45, 0x29, 0x3d, 0x38};
		unsigned char ref_tag[] = {0xc4, 0xf6, 0x74, 0xc0, 0xf6, 0x4b, 0x21, 0xe7, 0x37, 0x24, 0xdc, 0x76, 0xa6, 0x6c, 0x39, 0x19};

		ac_hb2_key(ctx, key, sizeof(key));
		ac_hb2_init(ctx, key, sizeof(key), iv, sizeof(iv), 0, 0);
		for (int j = 0; j < sizeof(msg); j += AC_HB2_BLOCK_LEN) {
			ac_hb2_enc(ctx, t2 + j, msg + j, AC_HB2_BLOCK_LEN);
		}
		ac_hb2_tag(ctx, tag, sizeof(tag));
		TEST_ASSERT(memcmp(t2, ref_cipher, sizeof(ref_cipher)) == 0, end);
		TEST_ASSERT(memcmp(tag, ref_tag, sizeof(ref_tag)) == 0, end);
	}
	TEST_END;

	code = STS_OK;
end:
	return code;
}

static int masha(void) {
	int code = STS_ERR;
	unsigned char key[] = {0x3c, 0x2d, 0x1e, 0x0f, 0x78, 0x69, 0x5a, 0x4b, 0xb4, 0xa5, 0x96, 0x87, 0xf0, 0xe1, 0xd2, 0xc3};
	unsigned char iv[] = {0xc0, 0xd0, 0xe0, 0xf0, 0x80, 0x90, 0xa0, 0xb0, 0x40, 0x50, 0x60, 0x70, 0xf0, 0x10, 0x20, 0x30, 0xb1, 0xc1, 0xd1, 0xe1, 0x71, 0x81, 0x91, 0xa1};
	align const unsigned char cipher_ref[] = {0x32, 0x68, 0x8c, 0xfe, 0x5f, 0x28, 0xda, 0xac, 0x9b, 0xe4, 0xe4, 0x6e, 0x81, 0x0f, 0xa1, 0x32, 0x38, 0x25, 0x17, 0xf3, 0xe1, 0x9b, 0x06, 0x67, 0xfa, 0x64, 0xb0, 0xc5, 0x74, 0x78, 0xab, 0x71, 0xca, 0x25, 0x72, 0x41, 0x07, 0xe2, 0x49, 0xce, 0x1d, 0x39, 0x23, 0x72, 0xc7, 0x28, 0xac, 0x90, 0xf0, 0x7c, 0x26, 0xc8, 0xc4, 0x34, 0x9d, 0xbb, 0x4c, 0x41, 0x4f, 0x30, 0xc0, 0xea, 0x7a, 0x70, 0xc4, 0xb8, 0xd0, 0x81, 0xa9, 0x49, 0x94, 0xb3, 0xed, 0xa9, 0x5c, 0x73, 0x7e, 0x9c, 0x56, 0xcb, 0x8d, 0xc6, 0x55, 0xee, 0xba, 0x3a, 0x4b, 0xb8, 0x25, 0xdc, 0x42, 0x4f, 0xf7, 0x3d, 0x97, 0x60, 0x96, 0x5a, 0x5b, 0xa9, 0xac, 0x48, 0x76, 0x49, 0xfa, 0xe0, 0xb0, 0xa7, 0xc8, 0xf4, 0x00, 0x7a, 0xcb, 0x3e, 0xa6, 0xdb, 0x37, 0xf3, 0xe2, 0x53, 0xd1, 0x66, 0xaf, 0x91, 0x3a, 0x00, 0x2e, 0x9e};
	const unsigned char mac_ref[] = {0x88, 0xfc, 0x3f, 0xe1, 0x43, 0xc9, 0x45, 0x48, 0x85, 0xaa, 0x7f, 0xb7, 0x23, 0x70, 0xf6, 0xb1, 0x0f, 0xbb, 0xab, 0x60, 0xc1, 0xc9, 0x0f, 0xd0, 0xf3, 0xc6, 0xa3, 0xf6, 0x0c, 0x18, 0x55, 0x56};
	unsigned char tag[AC_MASHA_TAG_LEN];
	uint32_t msg[128 / sizeof(uint32_t)];
	unsigned char ciphertext[128];
	unsigned char plaintext[128];
	int32_t i;
	ac_masha_ctx_t ctx;

	TEST_ONCE("MASHA passes test vector")
	{
		for (i = 0; i < (sizeof(msg) >> 2); i++) {
			msg[i] = ((i << 2) << 24) ^ (((i << 2) + 1) << 16)
					^ (((i << 2) + 2) << 8) ^ ((i << 2) + 3);
		}

		ac_masha_key(ctx, key, sizeof(key));
		ac_masha_init(ctx, key, sizeof(key), iv, sizeof(iv), 0, 0);

		for (i = 0; i < sizeof(msg); i += AC_MASHA_BLOCK_LEN) {
			ac_masha_enc(ctx, ciphertext + i, ((unsigned char *) msg) + i, AC_MASHA_BLOCK_LEN);
		}
		TEST_ASSERT(memcmp(ciphertext, cipher_ref, sizeof(ciphertext)) == 0, end);

		ac_masha_tag(ctx, tag, sizeof(tag));
		TEST_ASSERT(memcmp(tag, mac_ref, sizeof(tag)) == 0, end);

		ac_masha_key(ctx, key, sizeof(key));
		ac_masha_init(ctx, key, sizeof(key), iv, sizeof(iv), 0, 0);
		for (i = 0; i < sizeof(msg); i += AC_MASHA_BLOCK_LEN) {
			ac_masha_dec(ctx, plaintext + i, ciphertext + i, AC_MASHA_BLOCK_LEN);
		}
		TEST_ASSERT(ac_masha_check(ctx, tag, sizeof(tag)) == 1, end);
		TEST_ASSERT(memcmp(plaintext, msg, sizeof(plaintext)) == 0, end);
	}
	TEST_END;

	code = STS_OK;
  end:
	return code;
}

static int authenticated_cipher(void) {
	int code = STS_ERR;
	unsigned char key[BC_KEY_LEN];
	unsigned char iv[AC_IV_LEN];
	align unsigned char msg[AC_BLOCK_LEN];
	align unsigned char cipher[AC_BLOCK_LEN];
	align unsigned char computed_msg[AC_BLOCK_LEN];
	unsigned char tag[AC_TAG_LEN];
	ac_ctx_t ctx;

	TEST_BEGIN("AE is correct") {
		rand_bytes(key, sizeof(key));
		rand_bytes(iv, sizeof(iv));
		rand_bytes(msg, sizeof(msg));

		rand_bytes(cipher, sizeof(cipher));
		ac_key(ctx, key, sizeof(key));
		ac_init(ctx, key, sizeof(key), iv, sizeof(iv), AC_BLOCK_LEN, AC_BLOCK_LEN);
		ac_enc(ctx, cipher, msg, sizeof(msg));
		ac_data(ctx, msg, sizeof(msg));
		ac_tag(ctx, tag, sizeof(tag));

		rand_bytes(computed_msg, sizeof(computed_msg));
		ac_key(ctx, key, sizeof(key));
		ac_init(ctx, key, sizeof(key), iv, sizeof(iv), AC_BLOCK_LEN, AC_BLOCK_LEN);
		ac_dec(ctx, computed_msg, cipher, sizeof(msg));
		ac_data(ctx, msg, sizeof(msg));
		TEST_ASSERT(memcmp(msg, computed_msg, AC_BLOCK_LEN) == 0, end);
		TEST_ASSERT(ac_check(ctx, tag, sizeof(tag)) == 1, end);

		rand_bytes(computed_msg, sizeof(computed_msg));
		ac_key(ctx, key, sizeof(key));
		ac_init(ctx, key, sizeof(key), iv, sizeof(iv), AC_BLOCK_LEN, AC_BLOCK_LEN);
		ac_dec(ctx, computed_msg, cipher, sizeof(msg));
		ac_data(ctx, msg, sizeof(msg));
		rand_bytes(tag, sizeof(tag));
		TEST_ASSERT(ac_check(ctx, tag, sizeof(tag)) == 0, end);
	}
	TEST_END;

	for (int j = 1; j < AC_BLOCK_LEN; j++) {
		TEST_BEGIN("AE is correct for incomplete block") {
			rand_bytes(key, sizeof(key));
			rand_bytes(iv, sizeof(iv));
			rand_bytes(msg, sizeof(msg));

			rand_bytes(cipher, sizeof(cipher));
			ac_key(ctx, key, sizeof(key));
			ac_init(ctx, key, sizeof(key), iv, sizeof(iv), AC_BLOCK_LEN-j, AC_BLOCK_LEN-j);
			ac_enc(ctx, cipher, msg, sizeof(msg)-j);
			ac_data(ctx, msg, sizeof(msg)-j);
			ac_tag(ctx, tag, sizeof(tag));

			rand_bytes(computed_msg, sizeof(computed_msg));
			ac_key(ctx, key, sizeof(key));
			ac_init(ctx, key, sizeof(key), iv, sizeof(iv), AC_BLOCK_LEN-j, AC_BLOCK_LEN-j);
			ac_dec(ctx, computed_msg, cipher, sizeof(msg)-j);
			ac_data(ctx, msg, sizeof(msg)-j);
			TEST_ASSERT(memcmp(msg, computed_msg, AC_BLOCK_LEN-j) == 0, end);
			TEST_ASSERT(ac_check(ctx, tag, sizeof(tag)) == 1, end);
		}
		TEST_END;
	}

	code = STS_OK;

  end:
	return code;
}

int main(void) {
#ifdef __ICC430__
	// Stop watchdog timer to prevent time out reset
	WDTCTL = WDTPW + WDTHOLD;

	UCSCTL3 |= SELREF_2;// Set DCO FLL reference = REFO
	UCSCTL4 |= SELA_2;// Set ACLK = REFO

	__bis_SR_register(SCG0);// Disable the FLL control loop
	UCSCTL0 = 0x0000;// Set lowest possible DCOx, MODx
	UCSCTL1 = DCORSEL_7;// Select DCO range 24MHz operation
	UCSCTL2 = FLLD_1 + 609;// Set DCO Multiplier for 20 MHz
						   // (N + 1) * FLLRef = Fdco
						   // (609 + 1) * 32768 = 20 MHz
						   // Set FLL Div = fDCOCLK/2
	__bic_SR_register(SCG0);// Enable the FLL control loop

	// Worst-case settling time for the DCO when the DCO range bits have been
	// changed is n x 32 x 32 x f_MCLK / f_FLL_reference. See UCS chapter in 5xx
	// UG for optimization.
	// 32 x 32 x 20 MHz / 32,768 Hz = 375000 = MCLK cycles for DCO to settle
	__delay_cycles(625000);
#endif
	core_init();

	util_banner("Tests for the AC module:\n", 0);

	if (authenticated_cipher() != STS_OK) {
		core_clean();
		return 1;
	}

	if (ccm() != STS_OK) {
		core_clean();
		return 1;
	}

	if (gcm() != STS_OK) {
		core_clean();
		return 1;
	}

	if (ocb3() != STS_OK) {
		core_clean();
		return 1;
	}

	if (hb2() != STS_OK) {
		core_clean();
		return 1;
	}

	if (masha() != STS_OK) {
		core_clean();
		return 1;
	}

	core_clean();
}
#endif
